package com.filleduchaos.media_library

import android.provider.MediaStore
import java.io.File
import io.flutter.plugin.common.PluginRegistry.Registrar
import java.util.*
import java.util.logging.Logger

enum class MediaTypes {
  Images
}

class MediaLibrary(private val registrar: Registrar) {
  companion object {
    internal val imageColumnNames = arrayOf(
        MediaStore.Images.ImageColumns._ID,
        MediaStore.Images.ImageColumns.DATA,
        MediaStore.Images.Media.WIDTH,
        MediaStore.Images.Media.HEIGHT,
        MediaStore.Images.Media.LATITUDE,
        MediaStore.Images.Media.LONGITUDE,
        MediaStore.Images.Media.DATE_TAKEN,
        MediaStore.Images.Media.DATE_MODIFIED
    )
  }

  private val pathCache = mutableMapOf<String, String>()

  internal fun executeQuery(query: Map<String, Any>): ArrayList<Map<String, Any?>> {
    return when (query["type"]) {
      MediaTypes.Images.ordinal -> listImages(query)
      else -> throw IllegalArgumentException("Unsupported media type")
    }
  }

  private fun listImages(query: Map<String, Any>): ArrayList<Map<String, Any?>> {
    val logger = Logger.getLogger(registrar.activity().localClassName)
    val uri = MediaStore.Images.Media.EXTERNAL_CONTENT_URI
    val dateTaken = MediaStore.Images.ImageColumns.DATE_TAKEN
    val startDate = (query["addedDateStart"] as Long?)?.let { it / 1000 }
    val endDate = (query["addedDateEnd"] as Long?)?.let { it / 1000 }
    val shouldSelect = startDate != null && endDate != null
    val selection = if (shouldSelect) MediaStore.Images.Media.DATE_ADDED + " > ? AND " + MediaStore.Images.Media.DATE_ADDED + " < ?" else null
    val selectionArgs = if (shouldSelect) arrayOf(startDate.toString(), endDate.toString()) else null
    var sortOrder = "$dateTaken desc"
    (query["limit"])?.let { limit ->
      sortOrder += " limit $limit"
      (query["offset"])?.let { sortOrder += " offset $it" }
    }
//    (query["offset"])?.let { sortOrder += " offset $it"}

    val contentResolver = registrar.activity().contentResolver
    val imageList = ArrayList<Map<String, Any?>>()

    val cursor = contentResolver.query(uri, imageColumnNames, selection, selectionArgs, sortOrder)

    if (cursor.moveToFirst()) {
      logger.info("Reading columns!")
      val idColumn = cursor.getColumnIndexOrThrow(MediaStore.Images.ImageColumns._ID)
      val dataColumn = cursor.getColumnIndexOrThrow(MediaStore.Images.Media.DATA)
      val widthColumn = cursor.getColumnIndexOrThrow(MediaStore.Images.Media.WIDTH)
      val heightColumn = cursor.getColumnIndexOrThrow(MediaStore.Images.Media.HEIGHT)
      val latColumn = cursor.getColumnIndexOrThrow(MediaStore.Images.Media.LATITUDE)
      val longColumn = cursor.getColumnIndexOrThrow(MediaStore.Images.Media.LONGITUDE)
      val addedColumn = cursor.getColumnIndexOrThrow(MediaStore.Images.Media.DATE_TAKEN)
      val modifiedColumn = cursor.getColumnIndexOrThrow(MediaStore.Images.Media.DATE_MODIFIED)

      do {
        logger.info("Reading results!")
        val id = cursor.getString(idColumn)
        val path = cursor.getString(dataColumn)
        pathCache[id] = path
        val data = mutableMapOf<String, Any>("id" to id, "path" to path)

        cursor.getString(widthColumn)?.toLongOrNull()?.let { data["width"] = it }
        cursor.getString(heightColumn)?.toLongOrNull()?.let { data["height"] = it }
        cursor.getString(addedColumn)?.toLongOrNull()?.let { data["addedAt"] = it }
        cursor.getString(modifiedColumn)?.toLongOrNull()?.let { data["modifiedAt"] = it }

//        val latitude = cursor.getString(latColumn)?.toDoubleOrNull()
//        val longitude = cursor.getString(longColumn)?.toDoubleOrNull()


//        if (latitude != null && longitude != null) {
//          data["location"] = arrayOf<java.lang.Double>(latitude!, longitude!)
//        }


        cursor.getString(latColumn)?.toDoubleOrNull()?.let { latitude ->
          cursor.getString(longColumn)?.toDoubleOrNull()?.let { longitude ->
            data["latitude"] = latitude
            data["longitude"] = longitude
          }
        }

        imageList.add(data)
      } while (cursor.moveToNext())
    }
    cursor.close()

    return imageList
  }

  internal fun loadImage(id: String): ByteArray {
    val path = pathCache[id]

    path?.let {
      return File(it).readBytes()
    } ?: throw IllegalArgumentException("Un-cached id")
  }
}
