//
//  MediaLibrary.swift
//  media_library
//
//  Created by Pj Essien on 4/3/19.
//

import Foundation
import Photos

fileprivate enum MediaTypes: Int {
  case images
}

enum MediaLibraryError: Error {
  case unsupportedMediaType
  case offsetOutOfRange(offset: Int)
  case mediaNotFound(id: String)
  case couldNotLoad(id: String)
}

fileprivate func slice<T: PHObject>(of cursor: PHFetchResult<T>, query: [String: Any]) throws -> [T] {
  let count = cursor.count
  let start = (query["offset"] as! Int?) ?? 0

  if start >= count {
    throw MediaLibraryError.offsetOutOfRange(offset: start)
  }

  let limit = (query["limit"] as! Int?) ?? count
  let end = min(start + limit, count)
  let range = IndexSet.init(integersIn: start..<end)
  
  return cursor.objects(at: range)
}

public class MediaLibrary: NSObject {
//  private var photoCursor: PHFetchResult<PHAsset>?

  public func executeQuery(query: [String: Any]) throws -> [[String: Any]] {
    let queryType = query["type"] as! Int
    switch queryType {
    case MediaTypes.images.rawValue:
      return try listImages(query: query)
    default:
      throw MediaLibraryError.unsupportedMediaType
    }
  }
  
  func listImages(query: [String: Any]) throws -> [[String: Any]] {
//    if photoCursor == nil {
    let queryOptions = PHFetchOptions()
    queryOptions.sortDescriptors = [
      NSSortDescriptor(key: "creationDate", ascending: false)
    ]
    let addedStart = query["addedDateStart"] as! Int?
    let addedEnd = query["addedDateEnd"] as! Int?
    if let start = addedStart, let end = addedEnd {
      let startDate = NSDate(timeIntervalSince1970: Double(start / 1000))
      let endDate = NSDate(timeIntervalSince1970: Double(end / 1000))
      queryOptions.predicate = NSPredicate(format: "creationDate > %@ AND creationDate < %@", startDate, endDate)
    }
    
    let photoCursor = PHAsset.fetchAssets(with: PHAssetMediaType.image, options: queryOptions)
//    }
    
    return try slice(of: photoCursor, query: query).map({
      (asset: PHAsset) -> [String: Any] in
      var data: [String: Any] = [
        "id": asset.localIdentifier,
        "width": asset.pixelWidth,
        "height": asset.pixelHeight
      ]
      if let coords = asset.location?.coordinate {
        data["latitude"] = coords.latitude
        data["longitude"] = coords.longitude
      }
      if let addedAt = asset.creationDate {
        data["addedAt"] = addedAt.timeIntervalSince1970 * 1000
      }
      if let modifiedAt = asset.modificationDate {
        data["modifiedAt"] = modifiedAt.timeIntervalSince1970 * 1000
      }
      return data
    })
  }
  
  public func loadImage(id: String, done: @escaping (_ data: Data?) -> Void) throws {
    print("Loading image...")
    if let asset = PHAsset.fetchAssets(withLocalIdentifiers: [id], options: nil).firstObject {
      print(asset.localIdentifier)
      PHImageManager.default().requestImageData(for: asset, options: PHImageRequestOptions()) {
        (data, str, orientation, info) in
        print("Handler called")
        let error = info?[PHImageErrorKey] as? NSError
        if error != nil {
          print("An error!")
        }
        else {
          done(data)
        }
        
//        let error = info?[PHImageErrorKey] as? NSError
//        let cancelled = info?[PHImageCancelledKey] as? Bool
//        let degraded = info?[PHImageResultIsDegradedKey] as? Bool
//        if !(error != nil || cancelled! || degraded!) {
//          done(data!)
//        }
      }
    }
    else {
      throw MediaLibraryError.mediaNotFound(id: id)
    }
  }
}
